#!/usr/bin/python
import sys
try:
	from braille import *
except ImportError:
	from pylibbraille import *
except:
	print('You seem not to have libbraille installed. To use BrlView to view BrlTex BRF files on your braille display you need to install libbraille. You can get libbraille from www.libbraille.org. If you have libbraille installed, but still having trouble with BrlView then either upgrade python to 2.5 or install ctypes.')
	sys.exit()
def usage():
	print('BrlView 0.0.1')
	print('usage: BrlView.py (options) inputfile')
	print('')
	print('Options:')
	print('-b braille driver')
	print('-d device braille display is attached to')
	print('-h this help message')
if len(sys.argv) < 2:
	usage()
	sys.exit()
import getopt
try:
	opts, args = getopt.getopt(sys.argv[1:], 'b:d:h', ['braille=', 'device=', 'help'])
except getopt.GetoptError:
	usage()
	sys.exit()
brailledrivers = []
for x in range(0, braille_drivernum()):
	brailledrivers.append(braille_drivername(x))
for o, a in opts:
	if o in ['-b', '--braille']:
		if a in brailledrivers:
			braille_config(BRL_DRIVER, a)
		else:
			print('Braille driver not found. braille drivers are:')
			for x in brailledrivers:
				print(x)
			sys.exit()
	elif o in ['-d', '--device']:
		braille_config(BRL_DEVICE, a)
	elif o in ['-h', '--help']:
		usage()
		sys.exit()
braille_config(BRL_TABLE, 'american')
if not braille_init():
	print('problem initialising libbraille')
	print(braille_geterror())
	sys.exit()
displaylen = braille_size()
braille_timeout(-1)
key = brl_key()
#function to split those long lines
def linesplit(inputtext):
	'''Function to return a list of strings of the right length for the display'''
	outputlist = []
	while len(inputtext) > displaylen:
		outputlist.append(inputtext[:displaylen])
		inputtext = inputtext[displaylen:]
	outputlist.append(inputtext)
	return outputlist
for x in args:
	f = open(x)
	s = []
	for line in f:
		line = line.replace('\f', '')
		s.append(line)
	f.close()
	brlpos = 0
	x = 0
	brlline = linesplit(s[x])
	braille_display(brlline[0])
	keystatus = 1
	while keystatus == 1:
			braille_read(key)
			if key.type != 3:
				break
			if key.code == BRLK_UP:
				if x > 0:
					brlpos = 0
					x = x - 1
					brlline = linesplit(s[x])
					braille_display(brlline[0])
			elif key.code == BRLK_DOWN:
				if x < (len(s) - 1):
					brlpos = 0
					x += 1
					brlline = linesplit(s[x])
					braille_display(brlline[0])
			elif key.code == BRLK_BACKWARD:
				if brlpos > 0:
					brlpos = brlpos - 1
				elif (brlpos == 0) and (x > 0):
					x = x - 1
					brlline = linesplit(s[x])
					brlpos = len(brlline) -1
				braille_display(brlline[brlpos])
			elif key.code == BRLK_FORWARD:
				if len(brlline) - 1 > brlpos:
					brlpos += 1
				elif (len(brlline) - 1 == brlpos) and (len(s) -1 > x):
					x += 1
					brlpos = 0
					brlline = linesplit(s[x])
				braille_display(brlline[brlpos])
			else:
				break
braille_close()

