#!/usr/bin/python
#Copyright (C) 2006-2007 Michael Whapples, All Rights Reserved.
#Original Author: Michael Whapples, mwhapples@users.sourceforge.net
#Contributors: Alastair Irving,
#Unless linsor has explicitly granted you any other licensing terms, the contents of this file is subject to the RPL 
#1.1, 
#available at the BrlTex website (http://brltex.sourceforge.net).
#All software distributed under the Licenses is provided strictly on an "AS IS" basis,
#WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND the licensor
#HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
#OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT, OR NON-INFRINGEMENT.
#Licensor reserves the right to change the Venue stated in the license to the UK.

''''Renderer for plasTeX to give braille output'''
#
#This module will initially allow BAUK output
#The code for devices such as braille displays is computer braille, american for libbraille
import globalvars, string
import linewrap
import os.path
#Lets start with some variables that could be useful
#These are things like a math mode variable
class general:
	debug = False
	unknown = 0
class transdata:
	linelen = 40
	pagelen = 25
class _eqnnum:
	val = 0
#a cache for translations, should speed things up
class cache:
	translatetext = {}
	translatemath = {}
	translatemathcomp = {}
class transmode:
	compact = False
	mathmode = False
#A function to deal with items that are too long for the line
import sys
def longline(intext):
	breakline = -1
	#all maths symbols that can have a break after
	breakafchars = [']', ')', '>']
	#where to break,
	#make a list for each char, and choose the best
	breakpos = []
	for breakafch in breakafchars:
		while intext.find(breakafch, (breakline + 1), 40) > -1:
			breakline = intext.find(breakafch, (breakline + 1), 40)
		breakpos.append(breakline)
		breakline = -1
	breakpos.sort()
	if len(breakpos) > 0:
		intext = intext[breakpos[-1]] + '\n' + intext[breakpos[-1]+1]
	else:
		print('\nThere seems to be a line too long for you page, cannot sort this out automatically, exiting')
		sys.exit()
	return intext
#Initially define all the conversions
#the Renderer interfacing has to come at the end due to
#python requiring functions to be defined earlier than calls
#A function for generating the simple substitutions
def __makesubfunction__(subvalue):
	return lambda node: unicode(subvalue)
def simplesubs():
	f = open(os.path.join(globalvars.brltex_dir, 'bauk', 'simple.subs'))
	outDict = {}
	for line in f:
		if line.isspace():
			continue
		line = line.strip('\n')
		if len(line.split('\t')) == 2:
			outDict.update({line.split('\t')[0]: __makesubfunction__(line.split('\t')[1])})
		elif len(line.split('\t')) == 1:
			outDict.update({line.split('\t')[0]: __makesubfunction__(line.split('\t')[0])})
	f.close()
	return outDict
BAUKdict = {}
#Now some operators
#Note: some of these won't appear as nodes due to them being simple text
#An example of these text symbols are +, -, /, etc
def bracket_handler(node):
	try:
		return unicode(node.attributes['char'])
	except:
		return u''
BAUKdict.update({'Bigg': bracket_handler, 'bigg': bracket_handler, 
'Big': bracket_handler, 'big': bracket_handler})
def frac_handler(node):
	#Changing this to cope with fracs such int/int properly
	#Some of this was tidied up by Alastair Irving
	correctlen = False
	if (len(node.attributes['numer']) == 1) and (len(node.attributes['denom']) == 1):
		correctlen = True
	if unicode(node.attributes['numer'].textContent).isdigit() and unicode(node.attributes['denom'].textContent).isdigit() and correctlen:
		outtext = translate.numtrans(node.attributes["numer"].textContent)
		outtext += translate.lownumtrans(node.attributes["denom"].textContent)
		return unicode(outtext)
	else:
		numer = unicode(node.attributes['numer'])
		denom = unicode(node.attributes['denom'])
		for spacechar in string.whitespace:
			if spacechar in numer:
				numer = '<%s>' % numer
			if spacechar in denom:
				denom = '<%s>' % denom
		return u'<%s/%s>' % (numer, denom)
#The next bit may seem a bit out of place here, but these are used for the \over command
#In future when I have the time to really tweak things to be perfect I may do it differently 
#and not need this here
def bgroup_handler(node):
	#Work out if we have an over next to this one, if so leave it to the over function
	if (node.previousSibling is not None) and (node.previousSibling.nodeName == 'over'):
		return u''
	elif (node.nextSibling is not None) and (node.nextSibling.nodeName == 'over'):
		return u''
	#The brackets are probably not wanted out of equations,
	#anyway they would have to be different anyway if they were
	#There are some cases in equations also where they are not wanted
	if transmode.mathmode:
		if (len(node) == 1) and (len(node.textContent) == 1):
			return unicode(node)
		else:
			return u'<%s>' % unicode(node)
	else:
		return unicode(node)
def over_handler(node):
	#This will work out if it is a special case
	if (node.previousSibling.nodeName == 'bgroup') and (node.nextSibling.nodeName == 'bgroup'):
		if (len(node.previousSibling) == 1) and node.previousSibling.textContent.isdigit() and (len(node.nextSibling) == 1) and node.nextSibling.textContent.isdigit():
			return u'%s%s' % (translate.numtrans(node.previousSibling.textContent), translate.lownumtrans(node.nextSibling.textContent))
		return u'<%s/%s>' % (unicode(node.previousSibling), unicode(node.nextSibling))
	if (node.previousSibling.nodeName == 'bgroup'):
		return u'<%s>/' % unicode(node.previousSibling)
	elif node.nextSibling.nodeName == 'bgroup':
		return u'/<%s>' % unicode(node.nextSibling)
	return u'/'
def left_handler(node):
	return unicode(node.attributes['delim'])
def right_handler(node):
	return unicode(node.attributes['delim'])
#I don't have the braille to hand to confirm the translations for the dot commands that follow in the symbols file.
def re_handler(node):
	return u'$re%s' % unicode(node)
def im_handler(node):
	return u'$im%s' % unicode(node)
#leave these slightly rarer symbols for later.
#Not sure about surd, need to check
#The following may need to be tweaked, or other code may need a tweak
#to make it properly contract where possible to read correctly
#Hopefully this will be enough to be readable
def sqrt_handler(node):
	base = u''
	if node.attributes['self'] is not None:
		base = unicode(subscript_handler(node))
	content = unicode(node.attributes['text'])
	return u'%s%s%s' % (r'%', base, content)
#Again add this to the dictionary
BAUKdict.update({'frac': frac_handler, 'bgroup': bgroup_handler, 'over': 
over_handler,
'left': left_handler, 'right': right_handler,
'Re': re_handler, 'Im': im_handler,
'sqrt': sqrt_handler})
#moving on,
#Not sure if this is correct for arg, but seems sensible
#in the symbols file there is an example for log to base,
#this uses the log command, so should not repeat, the subscript will handle the base
#May as well do this sub/superscript bit here
def subscript_handler(node):
	#There are special cases where the general rule is broken, 
	#Now should be corrected
	#Again tidied by Alastair Irving
	correctlen = False
	outtext = u''
	transmode.compact = True
	if len(node.childNodes) == 1:
		correctlen = True
	if node.textContent.isdigit() and correctlen:
		outtext+=translate.lownumtrans(node.textContent)
	else:
		outtext = u'%s]' % unicode(node)
		if outtext[0] == u';':
			outtext = outtext[1:]
		outtext = u'*%s' % outtext
	transmode.compact = False
	return outtext
#read comments about subscript, same applies to superscript
def superscript_handler(node):
	#Again tidied by Alastair Irving
	correctlen = False
	outtext = u'+'
	transmode.compact = True
	if len(node.childNodes) == 1:
		correctlen = True
	if node.textContent.isdigit() and correctlen:
		outtext+=translate.lownumtrans(node.textContent)
	else:
		outtext = u'%s]' % unicode(node)
		if outtext[0] == ';':
			outtext = outtext[1:]
		outtext = '+%s' % outtext
	transmode.compact = False
	return outtext
#again the dictionary
BAUKdict.update({'active::_': subscript_handler, 'active::^': superscript_handler})
#Moving on to the accent part of the file
def hat_handler(node):
	#I think this is the right notation, will check though
	return u'%s?' % unicode(node)
def dot_handler(node):
	return u'%s\'' % unicode(node)
def ddot_handler(node):
	return u'%s-' % unicode(node)
def bar_handler(node):
	#This probably should check if it is one character
	return u'%s:' % unicode(node)
def vec_handler(node):
	#\vec may be used for vectors with arrows over the top, or as unit vectors
	#This decides which is so by the length
	if (len(node) == 1) and (len(node.textContent) == 1):
		if node.textContent.isupper():
			return u'^%s' % node.textContent
		elif node.textContent.islower:
			return u'@%s' % node.textContent
	return u'%s.3O' % unicode(node)
def mathbf_handler(node):
	#if (len(node) == 1) and (len(node.textContent) == 1) and node.textContent.isupper():
	#	return u'^%s' % node.textContent
	#elif (len(node) == 1) and (len(node.textContent) == 1) and node.textContent.islower:
	#	return u'@%s' % node.textContent
	#elif (len(node.textContent) >= 1) and node.textContent[0].isupper():
	#	#The following should have the second start at 2 to prevent the letter being done twice as liblouis inserts the capital sign
	#	return u'^%s%s' % (node.textContent[0], unicode(node)[2:])
	#elif (len(node.textContent) >= 1) and node.textContent.islower:
	#	return u'@%s%s' % (node.textContent[0], unicode(node)[2:])
	#return u'@%s' % unicode(node)
	#This is the improved version of mathbf, probably will handle the two if statements still active
	#I think those may work correctly, but if there seems to be some unusual output still possible to link to that then I will remove those as well.
	childnodelst = []
	#Add to the following list the nodeNames that should not get mathbf marks
	non_mathbf_list = ['cdot', 'times', 'prime']
	for childnode in node.childNodes.childNodes:
		if childnode.nodeName == '#text':
			#The following string should be used by characters which do not need 
			#bold marks before the entire thing is translated
			non_boldchars = ''
			for char in childnode:
				if char.isupper():
					childnodelst.append(non_boldchars)
					non_boldchars = ''
					childnodelst.append(u'^%s' % char)
				elif char.islower():
					childnodelst.append(non_boldchars)
					non_boldchars = ''
					childnodelst.append(u'@%s' % char)
				else:
					non_boldchars +=translate.math(char, transmode.compact)
			childnodelst.append(non_boldchars)
		else:
			#This will rely on authors being well behaved and not putting too much in mathbf,
			#This would not really work if they were to put another structure inside mathbf,
			#An example would be if a fraction were put in mathbf
			#I feel that example would be wrong anyway and should just be avoided from being used.
			if not childnode.nodeName in non_mathbf_list:
				childnodelst.append(u'@')
			childnodelst.append(unicode(BAUKdict[childnode.nodeName](childnode)))
	return u''.join(childnodelst)
def cal_handler(node):
	return unicode(node)
def mathrm_handler(node):
	return unicode(node)
BAUKdict.update({'bar': bar_handler, 'hat': hat_handler, 
'dot': dot_handler, 'ddot': ddot_handler,
'vec': vec_handler, 'mathbf': mathbf_handler, 'cal': cal_handler, 
'mathrm': mathrm_handler, 'widehat': vec_handler})
#Moving on again,
#there are some things in this that might be math dependent for braille output, 
#in these cases use the variable transmode.mathmode
def it_handler(node):
	#I think italics are done like this,
	#in fact I think all emphasised text should be done like this
	#so at the moment all bold, italic and underline will be pointed here
	if len(unicode(node)) > 1:
		return unicode('..%s' % unicode(node).replace(' ',' ..'))
	elif len(unicode(node)) == 1:
		return unicode('.%s' % unicode(node))
	else:
		return u''
BAUKdict.update({'bf': it_handler, 'it': it_handler,
'textit': it_handler, 'textbf': it_handler, 'bfseries': it_handler, 'emph': it_handler})
#The maths environments
def math_handler(node):
	transmode.mathmode = True
	outputtext = unicode(node)
	transmode.mathmode = False
	#It is wrong to have space at the end of math and at the beginning of text after math
	#only one is wanted, despose of the math space
	if len(outputtext) > 0 and outputtext[-1].isspace():
		if node.nextSibling.nodeName == '#text' and node.nextSibling.nodeValue[0] == ' ':
			outputtext = outputtext[:-1]
	return outputtext
def displaymath_handler(node):
	transmode.mathmode = True
	outputtext = unicode(node)
	transmode.mathmode = False
	return outputtext
def equation_handler(node):
	_eqnnum.val += 1
	transmode.mathmode = True
	outputtext = unicode(node)
	transmode.mathmode = False
	#outputtext += ' 7eq%s7' % translate.numtrans(_eqnnum.val)
	outputtext += ' 7eq%s7' % translate.numtrans(node.ref.childNodes[0])
	return outputtext
BAUKdict.update({'math': math_handler, 'displaymath': displaymath_handler, 'equation': equation_handler, 'equation*':   
displaymath_handler })
#Lets see to the paragraphs and sections
def par_handler(node):
	#There are certain cases where we don't want indent
	outputtext = ''
	noindent = ['tabular', 'array']
	if node.hasChildNodes():
		if node.childNodes[0].nodeName in noindent:
			return unicode('\n%s' % unicode(node))
	outputtext = unicode('\n  %s' % unicode(node))
	#This corrects if spaces have got in front of the beginning
	#while (len(outputtext) > 3) and outputtext[3].isspace():
		#outputtext = '\n  ' + outputtext[4:]
	return outputtext
def newline_handler(node):
	return u'\n'
def tab_handler(node):
	return u'  '
def newpage_handler(node):
	return u' \f'
def qquad_handler(node):
	return u''
def vspace_handler(node):
	return u''
def section_handler(node):
	sectionnames = ['chapter', 'section', 'subsection', 'subsubsection']
	outtext = '\n\n   %s %s' % (translate.text(u''.join(node.ref.childNodes)), node.attributes['title'])
	firstpar = True
	for par in node:
		if par.nodeName in sectionnames:
			outtext += section_handler(par)
		elif par.nodeName == 'par':
			if firstpar:
				outtext += u'\n'
			outtext += par_handler(par)
		else:
			outtext += u'\n\n%s' % unicode(par)
		firstpar = False
	#This corrects starting spacing
	while outtext[5].isspace():
		outtext = '\n\n   ' + outtext[6:]
	return outtext
#The document one,
#this should tidy up the ends
def document_handler(node):
	sectionnames = ['section', 'subsection', 'subsubsection', 'chapter']
	outtext = u''
	for section in node:
		if section.nodeName in sectionnames:
			outtext += section_handler(section)
		elif section.nodeName == 'par':
			outtext += par_handler(section)
			#outtext += u'\n  %s' % unicode(section)
		else:
			outtext += unicode(section)
	while outtext[0].isspace():
		outtext = outtext[1:]
	return outtext
#I think the default behaviour for tex is not to show anything on the \newcommand
def newcommand_handler(node):
	return u''
def abstract_handler(node):
	return u'   Abstract\n  %s' % unicode(node)
def verbatim_handler(node):
	#return translate.grade1(node.textContent, 'en-uk-g1.ctb')
	outtext = u''
	for line in node.textContent.split('\n'):
		outtext += translate.grade1(line, 'en-uk-verbatim.ctb')
		outtext += '\n'
	return outtext
def figure_handler(node):
	print('WARNING: BrlTex has encountered a image. BrlTex is unable to automatically process these images, so please manually create a suitable diagram.')
	outtext = unicode(node.getElementsByTagName('caption')[0])
	outtext = u'%s %s' % (translate.text('Figure caption reads:'), outtext)
	return outtext
def subsectionSubsubsectionList(subsectionlist):
	outtext = u''
	for subsection in subsectionlist:
		outtext += u'--%s %s\n' % (translate.text(u''.join(subsection.ref.childNodes)), unicode(subsection.tocEntry))
		subsubsectionlist = subsection.getElementsByTagName('subsubsection')
		if len(subsubsectionlist) > 0:
			for subsubsection in subsubsectionlist:
				outtext += u'----%s %s\n' % (translate.text(u''.join(subsubsection.ref.childNodes)), unicode(subsubsection.tocEntry))
	return outtext
def sectionSubsectionList(sectionlist):
	outtext = u''
	for section in sectionlist:
		outtext += u'%s %s\n' % (translate.text(u''.join(section.ref.childNodes)), unicode(section.tocEntry))
		subsectionlist = section.getElementsByTagName('subsection')
		if len(subsectionlist) > 0:
			outtext += subsectionSubsubsectionList(subsectionlist)
	return outtext
def tableofcontents_handler(node):
	outtext = translate.text('Table of contents') + u'\n'
	chapterlist = node.ownerDocument.getElementsByTagName('chapter')
	if len(chapterlist) > 0:
		for chapter in chapterlist:
			outtext += translate.text('Chapter%s %s\n' % (u''.join(chapter.ref.childNodes), unicode(chapter.tocEntry)))
			sectionlist = chapter.getElementsByTagName('section')
			if len(sectionlist) > 0:
				outtext += sectionSubsectionList(sectionlist)
	else:
		sectionlist = node.ownerDocument.getElementsByTagName('section')
		if len(sectionlist) > 0:
			outtext += sectionSubsectionList(sectionlist)
		else:
			subsectionlist = node.ownerDocument.getElementsByTagName('subsection')
			if len(subsectionlist) > 0:
				outtext += subsectionSubsubsectionList(subsectionlist)
			else:
				#Make this nothing then
				outtext = u''
	return outtext
def ref_handler(node):
	return translate.text('%s %s' % (node.idref['label'].counter, u''.join(node.idref['label'].ref.childNodes)))
def cite_handler(node):
	outtext = translate.text('refs:')
	for bibitem in node.bibitems:
		if bibitem.attributes['label'] == None:
			outtext += translate.text(bibitem.ref)
		else:
			outtext += unicode(bibitem.attributes['label'])
	return outtext
def thebibliography_handler(node):
	outtext = translate.text('Bibliography')
	outtext += u'\n'
	for bibitem in node:
		outtext += unicode(bibitem)
	return outtext
def layout_handler(node):
	outtext = unicode(node)
	translate.brlStop()
	return outtext
BAUKdict.update({'newline': newline_handler, 'par': par_handler, 'section': section_handler, 'subsection': 
section_handler, 'subsubsection': section_handler, 'chapter': section_handler,
'document': document_handler, 'newcommand': newcommand_handler, 
'qquad': qquad_handler, 'newpage': newpage_handler, ' ': tab_handler,
'vspace': vspace_handler, 'document-layout': layout_handler, 
'tableofcontents': tableofcontents_handler, 'abstract': 
abstract_handler, 'figure': figure_handler, 'verbatim': 
verbatim_handler, 'ref': ref_handler, 'cite': cite_handler, 
'thebibliography': thebibliography_handler})
#Now the title stuff
def title_handler(node):
	return u'   %s' % unicode(node)
def author_handler(node):
	if not unicode(node).isspace():
		return u'   ,AU?OR2 %s' % node.textContent
	return u''
def date_handler(node):
	return u'   ,DATE2 %s' % unicode(node).replace('\n', '')
def maketitle_handler(node):
	return u''
BAUKdict.update({'title': title_handler, 'author': author_handler, 'date': date_handler, 'maketitle': 
maketitle_handler})
#Now some lists
def enumerate_handler(node):
	outtext = ''
	itemcount = 1
	for item in node:
		outtext = outtext + translate.numtrans(itemcount) + ',4 '
		for par in item:
			outtext = outtext + unicode(par)
		outtext = outtext + '\n  '
		itemcount = itemcount + 1
	return outtext[:-3]
def itemize_handler(node):
	outtext = ''
	for item in node:
		for par in item:
			outtext = outtext + unicode(par)
		outtext = outtext + '\n  '
	return outtext[:-3]
BAUKdict.update({'enumerate': enumerate_handler, 'itemize': itemize_handler})
def table_handler(node):
	#This will handle a table environment
	#Simply give the table heading and give the table
	output=unicode(node.getElementsByTagName('caption')[0]) + '\n'
	output+=tabular_handler(node.getElementsByTagName('tabular')[0])
	return output
def tabular_handler(node):
	#We will need to get hold of the contents of each cell before we can output.
	#This will aid with processing,
	#to check things like the table will fit to the page,
	#If it won't fit, it should then try and do it in paragraph form
	#This will hold all cells
	cells = []
	#this will hold column lengths
	collen = [0]
	#Now get the cells
	for row in node:
		rows = []
		for cell in row:
			outtext = ''
			for par in cell:
				outtext = outtext + unicode(par)
			rows.append(outtext)
		#Check if the column lengths have changed
		if len(collen) != len(rows):
			collen = []
			for x in rows:
				collen.append(len(x))
		else:
			for x, y in enumerate(rows):
				if collen[x] < len(y):
					collen[x] = len(y)
		cells.append(rows)
	#Now find if it fits pagewidth
	tablewidth = 0
	for x in collen:
		tablewidth = tablewidth + x
	outtext = ''
	if tablewidth < transdata.linelen:
		#Lets try and layout this correctly
		for x in cells:
			for colnum, y in enumerate(x):
				outtext = outtext + y
				#Now pad it to the next column
				padlen = collen[colnum] - len(y) + 1
				for padcount in range(0,padlen -1):
					outtext = outtext + ' '
			outtext = outtext + '\n'
	else:
		print('\nWARNING: Table too big for page, putting in paragraph form')
		outtext = translate.text('The following table is done in paragraph form. Each row is a paragraph, and each cell of the row is seperated from the next by a ;')
		outtext = outtext + '\n'
		for x in cells:
			for y in x:
				outtext = outtext + y + '; '
			outtext = outtext + '\n'
	return unicode(outtext)
def align_handler(node):
	#I have seen this only used for equations,
	#so I will assume at the moment that it only is used for equations
	#This does not need to space out equations,
	#it would make the braille bulky,
	#and the value of that layout might be questionable
	#if you differ on this let me know
	transmode.mathmode = True
	outtext = u""
	for equation in node:
		for cell in equation:
			temptext = unicode(cell)
			#Strip the leading space
			while (len(temptext) != 0) and temptext[0].isspace():
				temptext = temptext[1:]
			#Now strip the trailing space
			while (len(temptext) != 0) and temptext[-1].isspace():
				temptext = temptext[:-1]
			outtext += temptext
		outtext += u'\n'
	#remove the last new line
	outtext = outtext[:-1]
	transmode.mathmode =False
	return outtext
BAUKdict.update({'table': table_handler, 'array': tabular_handler, 'tabular': tabular_handler,
'align*': align_handler, 'eqnarray*': align_handler})
#Now make the renderer part
import platform
from plasTeX.Renderers import Renderer
import translate
class brlrenderer(Renderer):
	def default(self, node):
		#print('<%s>' % node.source)
		#This will handle all that has not been defined in the dictionary
		#so something useful needs to be done with this
		#at the moment I will just get it to dump the plasTeX names in the output
		if general.debug == False:
			general.unknown += 1
			return u'%s' % node.source
		s = []
		s.append('<%s>' % node.nodeName)
		if node.hasAttributes():
			for key, value in node.attributes.items():
				if key == 'self':
					continue
				s.append(' %s=%s\n' % (key, unicode(value)))
		s.append(unicode(node))
		s.append('</%s>' % node.nodeName)
		return u'\n'.join(s)
	def textDefault(self, node):
		#This handles all the text
		#At the moment some simple translation
		#in the future this should be completed
		#this may use external translators
		try:
			nodetext = unicode(node)
		except AttributeError:
			#probably is blank
			return ""
		#unknownchars = {u'\u201C': '"', u'\u201D': '"', u'\u2018': "'", u'\u2019': "'"}
		#Do the replacements
		#for keychar, valchar in unknownchars.items():
		#	nodetext = nodetext.replace(keychar, valchar)
		#Exit if string is empty, translator will have problems if an empty string is passed.
		if len(nodetext) == 0:
			return u''
		if transmode.mathmode == True:
			while (len(nodetext) > 0) and nodetext[0].isspace():
				nodetext = nodetext[1:]
			if len(nodetext) == 0:
				return u''
			try:
				if (not transmode.compact) and (node.previousSibling is not None) and (translate.mathpreferspbf.find(nodetext[0]) >= 0):
					if (node.previousSibling.nodeName != 'left'):
						nodetext = ' %s' % nodetext
			except:
				pass
			if cache.translatemath.has_key(nodetext) and (not transmode.compact):
				translated = cache.translatemath[nodetext]
			elif cache.translatemathcomp.has_key(nodetext) and transmode.compact:
				translated = cache.translatemathcomp[nodetext]
			else:
				translated = translate.math(nodetext, transmode.compact)
				if transmode.compact:
					cache.translatemathcomp.update({nodetext: translated})
				else:
					cache.translatemath.update({nodetext: translated})
			return translated
		else:
			#Solve the case where mathmode is finished, then there is a space before a punctuation mark
			if len(nodetext) > 1 and nodetext[0].isspace() and nodetext[1] in '.,?:!' and node.previousSibling != None and node.previousSibling.nodeName == 'math':
				nodetext = nodetext[1:]
			if cache.translatetext.has_key(nodetext):
				translated = cache.translatetext[nodetext]
			else:
				translated = translate.text(nodetext)
				cache.translatetext.update({nodetext: translated})
			return translated
	def processFileContent(self, document, content):
		#Firstly get the line lengths correct
		#s = content.split('\n')
		#outputtext = ''
		#linenum = 1
		#pagenum = 1
		#for x in s:
		#	words = x.split(' ')
		#	outputline = ''
		#	for y in words:
		#		#check the line length, it is different for line1
		#		if linenum != 1:
		#			linelength = transdata.linelen
		#		else:
		#			linelength = transdata.linelen - 8
		#		#see to newpages which are specified in the source
		#		if y == '\f':
		#			y = ''
		#			#pad the line to full length
		#			while len(outputline) <= linelength:
		#				outputline += ' '
		#			#put in any page numbers
		#			if linenum == 1:
		#				pagenumstr = translate.numtrans(str(pagenum))
		#				while len(outputline) + len(pagenumstr) < transdata.linelen:
		#					outputline += ' '
		#				outputline += pagenumstr
		#			linenum = transdata.pagelen
		#		if (len(outputline) + len(y)) < transdata.linelen:
		#			outputline += y + ' '
		#		elif (len(outputline) + len(y)) == transdata.linelen:
		#			outputline += y
		#		else:
		#			if linenum != 1:
		#				outputtext += outputline + '\n'
		#				linenum += 1
		#			else:
		#				pagenumstr = translate.numtrans(str(pagenum))
		#				while len(outputline) + len(pagenumstr) < transdata.linelen:
		#					outputline += ' '
		#				outputline += pagenumstr
		#				outputtext += outputline + '\n'
		#			linenum += 1
		#			if linenum > transdata.pagelen:
		#				linenum = 1
		#				pagenum += 1
		#				outputtext += '\f'
		#			if y.isspace() or (y == ''):
		#				outputline = ''
		#			else:
		#				outputline = y + ' '
		#	if linenum != 1:
		#		outputtext += outputline + '\n'
		#	else:
		#		pagenumstr = translate.numtrans(str(pagenum))
		#		while len(outputline) + len(pagenumstr) < transdata.linelen:
		#			outputline += ' '
		#		outputtext += outputline + pagenumstr + '\n'
		#	linenum += 1
		#	if linenum > transdata.pagelen:
		#		linenum = 1
		#		pagenum += 1
		#		outputtext += '\f'
		wd = {';7': '"\n;7', ';6': '"\n;6', ';-': '"\n;-', ';8': '"\n;8', '_S': '\n_s', '>/<': '>"\n/<', '><': '>"\n<', ' ': '\n'}
		outputtext = linewrap.linewrap(content, transdata.linelen, wd, transdata.pagelen, translate.numtrans)
		#Now give those windows users the line endings for Windows
		if platform.system() == 'Windows':
			outputtext = outputtext.replace('\n', '\r\n')
		#translate.brlStop()
		return outputtext
BAUKdict.update(simplesubs())
def Renderer(brldict=BAUKdict):
	return brlrenderer(brldict)

